package net.wanji.utc.hisense.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.mapper.CrossPhaseMapper;
import net.wanji.databus.dao.mapper.CrossSchemeMapper;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.hisense.cache.CrossRunSchemeCache;
import net.wanji.utc.hisense.cache.CrossSchemePhaseTimeCountCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.pojo.CrossPhaseDirTurnPojo;
import net.wanji.utc.hisense.pojo.dto.CrossSchemePhaseCountDownDTO;
import net.wanji.utc.hisense.pojo.dto.PhaseCountDownDTO;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author duanruiming
 * @date 2023/09/20 14:25
 */
@Component
@Slf4j
public class HisensePhaseCountDownTask {

    @Resource
    private CrossPhaseMapper crossPhaseMapper;
    @Resource
    private CrossSchemeMapper crossSchemeMapper;

    @Scheduled(fixedRate = 1000, initialDelay = 1000)
    public void phaseTimeCountDown() {
        Map<String, LightsStatusVO> runningStateInfoCache = SignalDataCache.runningStateInfoCache;
        Map<String, Long> runningStatusStampMap = SignalDataCache.runningStatusStampMap;
        if (!runningStateInfoCache.isEmpty()) {
            for (Map.Entry<String, LightsStatusVO> entry : runningStateInfoCache.entrySet()) {
                String crossId = entry.getKey();
                LightsStatusVO lightsStatusVO = entry.getValue();
                String schemeStartTime = lightsStatusVO.getSchemeStartTime();
                Long lastPhaseTimeStamp = runningStatusStampMap.get(crossId);
                String phaseId = lightsStatusVO.getPhaseId();
                String schemeId = lightsStatusVO.getSchemeId();
                if (Objects.nonNull(lastPhaseTimeStamp) && !StringUtils.equalsIgnoreCase(schemeStartTime, String.valueOf(lastPhaseTimeStamp))) {
                    CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, Integer.valueOf(schemeId));
                    // 通过时段表，切换方案 凌晨00：00 如果切换下一个日期方案，可能RefreshCacheTask还没执行
                    String nextSectionSchemeNo = CrossRunSchemeCache.currentRunSchemeNoCache.get(crossId);
                    String currentTime = String.valueOf(new Date().getTime());
                    if (StringUtils.equalsIgnoreCase(schemeStartTime, currentTime)) {
                        crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, Integer.valueOf(nextSectionSchemeNo));
                        executeNextSectionScheme(crossId, lightsStatusVO, "1", crossSchemePO);
                        lightsStatusVO.setCycleLen(crossSchemePO.getCycle());
                    } else {
                        executeNextPeriod(crossId, lightsStatusVO, phaseId, crossSchemePO);
                    }
                    // 当前方案下一周期
                    setDirLampGroupMap(lightsStatusVO);
                    runningStateInfoCache.put(crossId, lightsStatusVO);
                }
            }
        }
    }

    /**
     * @description 通过日计划缓存，切换下一个时段方案
     * @param crossId
     * @param lightsStatusVO
     * @param phaseId
     * @param crossSchemePO
     */
    private void executeNextSectionScheme(String crossId, LightsStatusVO lightsStatusVO, String phaseId, CrossSchemePO crossSchemePO) {
        if (Objects.nonNull(crossSchemePO)) {
            List<CrossPhasePO> nextSchemePhaseList = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
            Integer phaseTime = 0;
            if (!CollectionUtils.isEmpty(nextSchemePhaseList)) {
                for (CrossPhasePO crossPhasePO : nextSchemePhaseList) {
                    if (StringUtils.equalsIgnoreCase(phaseId, crossPhasePO.getPhaseNo())) {
                        phaseTime = crossPhasePO.getPhaseTime();
                    }
                }
            }
            lightsStatusVO.setPhaseId(phaseId);
            lightsStatusVO.setCyclePhaseCountDown(phaseTime);
        }
    }

    /**
     * @description 切换下一方案周期
     * @param crossId
     * @param lightsStatusVO
     * @param phaseId
     * @param crossSchemePO
     */
    private void executeNextPeriod(String crossId, LightsStatusVO lightsStatusVO, String phaseId, CrossSchemePO crossSchemePO) {
        if (lightsStatusVO.getCyclePhaseCountDown() == 0 && Objects.nonNull(crossSchemePO)) {
            List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
            Integer nextPhaseTime = 0;
            String nextPhaseId = "0";
            if (!CollectionUtils.isEmpty(crossPhasePOS)) {
                for (int i = 0; i < crossPhasePOS.size(); i++) {
                    if (StringUtils.equalsIgnoreCase(phaseId, crossPhasePOS.get(i).getPhaseNo())) {
                        if (i == crossPhasePOS.size() - 1) {
                            nextPhaseTime = crossPhasePOS.get(0).getPhaseTime();
                            nextPhaseId = "1";
                        } else {
                            nextPhaseTime = crossPhasePOS.get(i + 1).getPhaseTime();
                            nextPhaseId = crossPhasePOS.get(i + 1).getPhaseNo();
                        }
                    }
                }
            }
            lightsStatusVO.setPhaseId(nextPhaseId);
            lightsStatusVO.setCyclePhaseCountDown(nextPhaseTime);
        } else {
            lightsStatusVO.setCyclePhaseCountDown(lightsStatusVO.getCyclePhaseCountDown() - 1);
        }
    }

    /**
     * @description 通过当前周期倒计时方案相位灯态
     * @param lightsStatusVO
     */
    public void setDirLampGroupMap(LightsStatusVO lightsStatusVO) {
        String crossId = lightsStatusVO.getCrossId();
        String currentSchemeNo = lightsStatusVO.getSchemeId();
        String currentPhaseNo = lightsStatusVO.getPhaseId();
        Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        if (!CollectionUtils.isEmpty(crossPhaseDirTurnCache)) {
            Map<String, Object> dirLampGroupMap = new HashMap<>();
            for (CrossPhaseDirTurnPojo phaseDirTurnPojo : crossPhaseDirTurnCache) {
                String schemeNo = String.valueOf(phaseDirTurnPojo.getSchemeNo());
                String phaseNo = String.valueOf(phaseDirTurnPojo.getPhaseNo());
                if (StringUtils.equalsIgnoreCase(schemeNo, currentSchemeNo) && StringUtils.equalsIgnoreCase(currentPhaseNo, String.valueOf(phaseNo))) {
                    PhaseCountDownDTO phaseCountDownDTO = getPhaseCountDownDTO(crossId, schemeNo, phaseNo);
                    Integer yellowTime = phaseCountDownDTO.getYellowTime();
                    Integer redTime = phaseCountDownDTO.getRedTime();
                    Map<Integer, List<CrossLightsPO>> dirTurnMap = phaseDirTurnPojo.getDirTurnMap();
                    if (!dirTurnMap.isEmpty()) {
                        for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
                            Integer dir = dirEntry.getKey();
                            List<CrossLightsPO> lightsList = dirEntry.getValue();
                            if (CollectionUtils.isEmpty(lightsList)) {
                                continue;
                            }
                            Map<String, String> turnColor = new HashMap<>();
                            Integer type = 2; // 1 圆饼灯
                            String color = null;
                            for (CrossLightsPO crossLightsPO : lightsList) {
                                type = crossLightsPO.getType();
                                Integer turn = crossLightsPO.getTurn();
                                String turnStr = String.valueOf(turn);
                                if (cyclePhaseCountDown > yellowTime + redTime) {
                                    color = "green";
                                    if (turn == 20) {
                                        turnColor.put(turnStr, "red");
                                    }
                                }
                                if (cyclePhaseCountDown <= yellowTime + redTime && cyclePhaseCountDown > redTime) {
                                    color = "yellow";
                                    if (turn == 20) {
                                        turnColor.put(turnStr, "red");
                                    }
                                }
                                if (cyclePhaseCountDown < redTime) {
                                    color = "red";
                                    if (turn == 20) {
                                        turnColor.put(turnStr, "green");
                                    }
                                }
                                turnColor.put(turnStr, color);
                            }
                            setAllTurnColor(turnColor, type, color);
                            dirLampGroupMap.put(String.valueOf(dir), turnColor);
                        }
                    }
                    lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
                }
            }
        }
    }

    private static void setAllTurnColor(Map<String, String> turnColor, Integer type, String color) {
        List<String> allTurn = Arrays.asList("1", "2", "3");
        for (String turn : allTurn) {
            if (!turnColor.containsKey(turn)) {
                if (Objects.equals(type, 1)) { // 圆饼灯 相同颜色
                    turnColor.put(turn, color);
                } else {
                    if ("red".equals(color)) { // 箭头灯 相反
                        turnColor.put(turn, "green");
                    } else { // green yellow 都是黄色
                        turnColor.put(turn, "red");
                    }
                }
            }
        }
    }

    /**
     * @description 获取路口方案相位倒计时
     * @param crossId
     * @param schemeNo
     * @param phaseNo
     * @return
     */
    private PhaseCountDownDTO getPhaseCountDownDTO(String crossId, String schemeNo, String phaseNo) {
        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                if (Objects.equals(crossId, dto.getCrossId()) && Objects.equals(schemeNo, dto.getSchemeNo())) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    if (!CollectionUtils.isEmpty(phaseCountDownDTOList)) {
                        for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownDTOList) {
                            if (Objects.equals(phaseNo, phaseCountDownDTO.getPhaseNo())) {
                                return phaseCountDownDTO;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }
}
